/**
 * File for load simulation logic.
 */


// User Import Headers.
#ifndef user_headers
    #define user_headers
    #include "simulate_loads.h"
    #include "structs.h"
#endif


/**
 * Logic to run "Asynchronous Round Robin" load scheme.
 */
void run_arr(int total_processors, int seconds_per_index, int indexes_per_load, int total_loads) {
    printf("Starting ARR load scheme.");
    printf("\n");

    // Initialize threading.
    pthread_t* thread_pool = calloc((total_processors - 1), sizeof(pthread_t));
    int* thread_results = calloc((total_processors - 1), sizeof(int));

    // Launch threads.
    for (int index = 0; index < (total_processors - 1); index++) {

        // // Initialize thread args.
        thread_struct* thread_args = initialize_thread_struct(
            &thread_pool, &thread_results, total_processors, seconds_per_index, indexes_per_load, total_loads, index
        );

        // Create current thread.
        pthread_create(&thread_pool[index], NULL, simulate_arr_load, (void*) thread_args);
    }

    // Wait for threads to finish.
    for (int index = 0; index < (total_processors - 1); index++) {
        pthread_join(thread_pool[index], NULL);
    }

    // Free variables.
    free(thread_pool);
    free(thread_results);

    printf("\n");
    printf("ARR load scheme. Finished.");
}


/**
 * Logic to run "Global Round Robin" load scheme.
 */
void run_grr(int total_processors) {
    printf("Starting GRR load scheme.");
    printf("\n");

    pthread_t* thread_pool = NULL;

    printf("\n");
    printf("GRR load scheme. Finished.");
}


/**
 * Logic to run "Random Poling" load scheme.
 */
void run_rp(int total_processors) {
    printf("Starting RP load scheme.");
    printf("\n");

    pthread_t* thread_pool = NULL;

    printf("\n");
    printf("RP load scheme. Finished.");
}



/**
 * Logic to run "Nearest Neighbor" load scheme.
 */
void run_nn(int total_processors) {
    printf("Starting NN load scheme.");
    printf("\n");

    pthread_t* thread_pool = NULL;

    printf("\n");
    printf("NN load scheme. Finished.");
}


// /**
//  * Function to simulate working on a load.
//  */
// void simulate_load(char* thread_name) {
//     printf("Simulating load processing on thread \"%s\".\n", thread_name);

//     int load_counter = 0;
//     int index = 0;

//     // Iterate through loads. Each load has indexes to simulate "work".
//     while (load_counter < total_loads) {

//         // Increment load num.
//         load_counter += 1;
//         printf("Simulating load %i on thread \"%s\".\n", load_counter, thread_name);

//         // Simulate running the load.
//         index = 0;
//         while (index < indexes_per_load) {

//             // Print out index value.
//             if ((index % 10) == 0) {
//                 printf("load %i / %i    index %i / %i\n", load_counter, total_loads, index, indexes_per_load);
//             }

//             // Wait 1 second to simulate work.
//             sleep(1);

//             // Increment index.
//             index += 1;
//         }

//     }

// }


/**
 * Function to simulate working on an ARR load.
 */
void* simulate_arr_load(thread_struct* thread_struct_ptr) {
    printf("Simulating load processing on thread \"%i\".\n", thread_struct_ptr->thread_num);

    // int load_counter = 0;
    // int index = 0;

    // // Iterate through loads. Each load has indexes to simulate "work".
    // while (load_counter < total_loads) {

    //     // Increment load num.
    //     load_counter += 1;
    //     printf("Simulating load %i on thread \"%s\".\n", load_counter, thread_name);

    //     // Simulate running the load.
    //     index = 0;
    //     while (index < indexes_per_load) {

    //         // Print out index value.
    //         if ((index % 10) == 0) {
    //             printf("load %i / %i    index %i / %i\n", load_counter, total_loads, index, indexes_per_load);
    //         }

    //         // Wait 1 second to simulate work.
    //         sleep(1);

    //         // Increment index.
    //         index += 1;
    //     }

    // }

    free(thread_struct_ptr);
}

